@@ -93,7 +93,7 @@ class App():
         #
         # Earily initialization
         #
-        is_a_tty = Terminal().is_a_tty
+        is_a_tty = sys.stdout.isatty() and sys.stderr.isatty()
 
         # Enable interactive mode if we're attached to a tty
         if main_options['no_interactive']:
