@@ -1,5 +1,5 @@
 #
-#  Copyright (C) 2016 Codethink Limited
+#  Copyright (C) 2018 Codethink Limited
 #
 #  This program is free software; you can redistribute it and/or
 #  modify it under the terms of the GNU Lesser General Public
@@ -22,6 +22,7 @@ import collections
 import string
 from copy import deepcopy
 from contextlib import ExitStack
+from pathlib import Path
 
 from ruamel import yaml
 from ruamel.yaml.representer import SafeRepresenter, RoundTripRepresenter
@@ -37,6 +38,19 @@ RoundTripConstructor.add_constructor(u'tag:yaml.org,2002:float', RoundTripConstr
 PROVENANCE_KEY = '__bst_provenance_info'
 
 
+# Provides information about file for provenance
+#
+# Args:
+#    name (str): Full path to the file
+#    shortname (str): Relative path to the file
+#    project (Project): Project where the shortname is relative from
+class ProvenanceFile():
+    def __init__(self, name, shortname, project):
+        self.name = name
+        self.shortname = shortname
+        self.project = project
+
+
 # Provenance tracks the origin of a given node in the parsed dictionary.
 #
 # Args:
@@ -56,7 +70,7 @@ class Provenance():
 
     # Convert a Provenance to a string for error reporting
     def __str__(self):
-        return "{} [line {:d} column {:d}]".format(self.filename, self.line, self.col)
+        return "{} [line {:d} column {:d}]".format(self.filename.shortname, self.line, self.col)
 
     # Abstract method
     def clone(self):
@@ -174,13 +188,15 @@ class CompositeTypeError(CompositeError):
 #
 # Raises: LoadError
 #
-def load(filename, shortname=None, copy_tree=False):
+def load(filename, shortname=None, copy_tree=False, *, project=None):
     if not shortname:
         shortname = filename
 
+    file = ProvenanceFile(filename, shortname, project)
+
     try:
         with open(filename) as f:
-            return load_data(f, shortname=shortname, copy_tree=copy_tree)
+            return load_data(f, file, copy_tree=copy_tree)
     except FileNotFoundError as e:
         raise LoadError(LoadErrorReason.MISSING_FILE,
                         "Could not find file at {}".format(filename)) from e
@@ -192,7 +208,7 @@ def load(filename, shortname=None, copy_tree=False):
 
 # Like load(), but doesnt require the data to be in a file
 #
-def load_data(data, shortname=None, copy_tree=False):
+def load_data(data, file=None, copy_tree=False):
 
     try:
         contents = yaml.load(data, yaml.loader.RoundTripLoader, preserve_quotes=True)
@@ -207,9 +223,9 @@ def load_data(data, shortname=None, copy_tree=False):
         else:
             raise LoadError(LoadErrorReason.INVALID_YAML,
                             "YAML file has content of type '{}' instead of expected type 'dict': {}"
-                            .format(type(contents).__name__, shortname))
+                            .format(type(contents).__name__, file.name))
 
-    return node_decorated_copy(shortname, contents, copy_tree=copy_tree)
+    return node_decorated_copy(file, contents, copy_tree=copy_tree)
 
 
 # Dumps a previously loaded YAML node to a file
@@ -392,6 +408,95 @@ def node_get(node, expected_type, key, indices=None, default_value=_get_sentinel
     return value
 
 
+# node_get_project_path()
+#
+# Fetches a project path from a dictionary node and validates it
+#
+# Paths are asserted to never lead to a directory outside of the project
+# directory. In addition, paths can not point to symbolic links, fifos,
+# sockets and block/character devices.
+#
+# The `check_is_file` and `check_is_dir` parameters can be used to
+# perform additional validations on the path. Note that an exception
+# will always be raised if both parameters are set to ``True``.
+#
+# Args:
+#    node (dict): A dictionary loaded from YAML
+#    key (str): The key whose value contains a path to validate
+#    project_dir (str): The project directory
+#    check_is_file (bool): If ``True`` an error will also be raised
+#                          if path does not point to a regular file.
+#                          Defaults to ``False``
+#    check_is_dir (bool): If ``True`` an error will be also raised
+#                         if path does not point to a directory.
+#                         Defaults to ``False``
+# Returns:
+#    (str): The project path
+#
+# Raises:
+#    (LoadError): In case that the project path is not valid or does not
+#                 exist
+#
+def node_get_project_path(node, key, project_dir, *,
+                          check_is_file=False, check_is_dir=False):
+    path_str = node_get(node, str, key)
+    path = Path(path_str)
+    project_dir_path = Path(project_dir)
+
+    provenance = node_get_provenance(node, key=key)
+
+    if (project_dir_path / path).is_symlink():
+        raise LoadError(LoadErrorReason.PROJ_PATH_INVALID_KIND,
+                        "{}: Specified path '{}' must not point to "
+                        "symbolic links "
+                        .format(provenance, path_str))
+
+    if path.parts and path.parts[0] == '..':
+        raise LoadError(LoadErrorReason.PROJ_PATH_INVALID,
+                        "{}: Specified path '{}' first component must "
+                        "not be '..'"
+                        .format(provenance, path_str))
+
+    try:
+        if sys.version_info[0] == 3 and sys.version_info[1] < 6:
+            full_resolved_path = (project_dir_path / path).resolve()
+        else:
+            full_resolved_path = (project_dir_path / path).resolve(strict=True)
+    except FileNotFoundError:
+        raise LoadError(LoadErrorReason.MISSING_FILE,
+                        "{}: Specified path '{}' does not exist"
+                        .format(provenance, path_str))
+
+    is_inside = project_dir_path in full_resolved_path.parents or (
+        full_resolved_path == project_dir_path)
+
+    if path.is_absolute() or not is_inside:
+        raise LoadError(LoadErrorReason.PROJ_PATH_INVALID,
+                        "{}: Specified path '{}' must not lead outside of the "
+                        "project directory"
+                        .format(provenance, path_str))
+
+    if full_resolved_path.is_socket() or (
+            full_resolved_path.is_fifo() or
+            full_resolved_path.is_block_device()):
+        raise LoadError(LoadErrorReason.PROJ_PATH_INVALID_KIND,
+                        "{}: Specified path '{}' points to an unsupported "
+                        "file kind"
+                        .format(provenance, path_str))
+
+    if check_is_file and not full_resolved_path.is_file():
+        raise LoadError(LoadErrorReason.PROJ_PATH_INVALID_KIND,
+                        "{}: Specified path '{}' is not a regular file"
+                        .format(provenance, path_str))
+
+    if check_is_dir and not full_resolved_path.is_dir():
+        raise LoadError(LoadErrorReason.PROJ_PATH_INVALID_KIND,
+                        "{}: Specified path '{}' is not a directory"
+                        .format(provenance, path_str))
+
+    return path_str
+
+
 # node_items()
 #
 # A convenience generator for iterating over loaded key/value
@@ -416,7 +521,7 @@ def node_items(node):
 def ensure_provenance(node):
     provenance = node.get(PROVENANCE_KEY)
     if not provenance:
-        provenance = DictProvenance('', node, node)
+        provenance = DictProvenance(ProvenanceFile('', '', None), node, node)
     node[PROVENANCE_KEY] = provenance
 
     return provenance
